require 'net/http'
require 'resultable'
require "ostruct"
require "pathname"
require "yaml"
require "erb"
require 'json'

module Buyatabwrapper
  class API
    include Resultable

    CALLS = {
      get_merchants: {
        url: '/gcp/services/BuyatabWS.asmx/GetMerchantList',
        default_params: {}
      },
      purchase_card: {
        url: '/gcp/services/BuyatabWS.asmx/PurchaseCard',
        default_params: {
          cardstyle_id: '0',
          msg: '',
          quantity: '1',
          recipient_name: '',
          recipient_email: '',
          sender_name: '',
          sender_email: '',
          handle_delivery: 'true',
          customer_ref_id: '',
          opt_in: 'true'
        }
      }
    }

    attr_accessor :result, :config

    def initialize
      self.result = Result.new
      self.config = parse_config_file
    end

    def send_request(type, params)
      return result unless config

      request = Net::HTTP::Post.new(CALLS[type][:url], initheader = {'Content-Type' => 'application/json'})
      request.body = default_params(type).merge(params).to_json

      http = Net::HTTP.new(config.host, 443).tap do |http|
        http.use_ssl = true
        http.verify_mode = OpenSSL::SSL::VERIFY_NONE
      end
      response = http.request(request)
      response = JSON.parse(response.body)["d"]

      if response["Status"]["Success"]
        if type == :get_merchants
          self.result = response['Merchants']
        else
          result.response = OpenStruct.new(
            id: response['BATOrderNumber'],
            url: response['Links'].first,
            card_number: response['CardNumbers'].first,
            pin_number: response['CardPins'].first
          )
          result.successful! 
        end
      else
        result.response = response["Status"]['Error']['Message']
      end

      result
    end

    private

    def default_params(type)
      {
        username: config.user_name, 
        password: config.password,
        apitoken: config.api_token
      }.merge(CALLS[type][:default_params])
    end

    def parse_config_file
      file = Rails.root.join("config").join("buyatabwrapper.yml")
      data = YAML.load(ERB.new(IO.read(file)).result)[Rails.env] rescue nil

      result.response = 'Provide configs in buyatabwrapper.yml file' unless data

      data ? OpenStruct.new(data) : data
    end
  end
end

